import pandas as pd
import xarray as xr

from ._resources import fetch_prudence
from ._regions import WGS84


def _create_polygons(df):
    from shapely.geometry import Polygon

    return [Polygon(_get_vertices(df, area)) for area in df.index]


def _get_vertices(df, area):
    coords = df.loc[area]
    return [
        [coords.west, coords.south],
        [coords.east, coords.south],
        [coords.east, coords.north],
        [coords.west, coords.north],
    ]


def _create_region(df, area):
    import regionmask

    polygon = _get_vertices(df, area)
    return regionmask.Regions([polygon])


def regions(df):
    import regionmask

    regions = [_get_vertices(df, area) for area in df.index]
    return regionmask.Regions(
        regions, names=df.name, abbrevs=df.index, name="prudence regions"
    )


def geodataframe(df, crs=WGS84):
    import geopandas as gpd

    return gpd.GeoDataFrame(
        df.drop(["west", "south", "east", "north"], axis=1),
        index=df.index,
        crs=crs,
        geometry=_create_polygons(df),
    )


def mask_3D(regions, lon, lat, **kwargs):
    regs = regions
    masks = [regs[[i]].mask_3D(lon, lat, **kwargs) for i in regs.numbers]
    return xr.concat(masks, dim="region")


class Prudence:
    """Prudence regions in Europe.

    Prediction of Regional scenarios and Uncertainties for Defining EuropeaN
    Climate change risks and Effects.
    """

    @property
    def df(self):
        return pd.read_csv(fetch_prudence(), na_filter=False, index_col="area")

    @property
    def geodataframe(self):
        return geodataframe(self.df)

    @property
    def regionmask(self):
        return regions(self.df)

    def mask_3D(self, lon, lat, **kwargs):
        return mask_3D(self.regionmask, lon, lat, **kwargs)


prudence = Prudence()



# -*- coding: utf-8 -*-
# flake8: noqa

import numpy as np

LAT_NAME = "lat"
LON_NAME = "lon"
RLAT_NAME = "rlat"
RLON_NAME = "rlon"


coords = {
    "rlon": {
        "axis": "X",
        "standard_name": "grid_longitude",
        "long_name": "longitude in rotated pole grid",
        "units": "degrees",
    },
    "rlat": {
        "axis": "Y",
        "standard_name": "grid_latitude",
        "long_name": "latitude in rotated pole grid",
        "units": "degrees",
    },
    "lon": {
        "standard_name": "longitude",
        "long_name": "longitude",
        "units": "degrees_east",
    },
    "lat": {
        "standard_name": "latitude",
        "long_name": "latitude",
        "units": "degrees_north",
    },
}


mapping = {
    "grid_mapping_name": "rotated_latitude_longitude",
    "grid_north_pole_latitude": 90.0,
    "grid_north_pole_longitude": 180.0,
    "north_pole_grid_longitude": 0.0,
}


DEFAULT_MAPPING_NCVAR = "rotated_latitude_longitude"


DEFAULT_CORDEX_ATTRS = {
    "institution": "",
    "institute_id": "",
    "experiment_id": "",
    "source": "",
    "model_id": "",
    "contact": "",
    "comment": "",
    "references": "",
    "initialization_method": 0,
    "physics_version": 0,
    "tracking_id": "",
    "CORDEX_domain": "",
    "driving_experiment": ", , ",
    "driving_model_id": "",
    "driving_model_ensemble_member": "r0i0p0",
    "driving_experiment_name": "",
    "rcm_version_id": "v1",
    "product": "output",
    "experiment": "",
    "frequency": "",
    "creation_date": "",
    "Conventions": "CF-1.4",
    "project_id": "CORDEX",
    "table_id": "",
    "title": "",
    "modeling_realm": "atmos",
    "realization": 0,
    "cmor_version": "",
}



"""Unit test package for cordex."""

import importlib

import pytest


def _importorskip(modname):
    try:
        importlib.import_module(modname)
        has = True
    except ImportError:  # pragma: no cover
        has = False
    func = pytest.mark.skipif(not has, reason=f"requires {modname}")
    return has, func


has_matplotlib, requires_matplotlib = _importorskip("matplotlib")
has_cartopy, requires_cartopy = _importorskip("cartopy")



"""this module should help on managing ESGF metadata
"""

import pandas as pd

try:
    from tqdm import tqdm
except:

    def tqdm(x):
        return x


try:
    from pyesgf.search import SearchConnection
except:
    print(
        "pyesgf client is not installed! please install https://github.com/ESGF/esgf-pyclient to make use of this module."
    )


DKRZ_HOST = "esgf-data.dkrz.de"
DKRZ_ESG_URL = "https://{}/esg-search".format(DKRZ_HOST)

# cordex  output  EUR-11  GERICS  ECMWF-ERAINT  evaluation  r1i1p1  REMO2015  v1  day  tasmax  v20180813
CORDEX_COLUMNS = [
    "project_id",
    "product",
    "domain",
    "institute",
    "driving_model_id",
    "experiment_id",
    "member",
    "model_id",
    "version",
    "frequency",
    "variable",
    "date",
    "host",
]
CMIP5_COLUMNS = [
    "project_id",
    "product",
    "domain",
    "institute",
    "driving_model_id",
    "experiment_id",
    "member",
    "model_id",
    "version",
    "frequency",
    "variable",
    "date",
]
CMIP6_COLUMNS = [
    "project_id",
    "product",
    "domain",
    "institute",
    "driving_model_id",
    "experiment_id",
    "member",
    "model_id",
    "version",
    "frequency",
    "variable",
    "date",
]


def logon(host=DKRZ_HOST):
    """logon to ESGF Host."""
    from pyesgf.logon import LogonManager

    print("logon to: ".format(host))
    lm = LogonManager()
    lm.logoff()
    lm.logon(hostname=host, interactive=True, bootstrap=True)
    print("logged on: {}".format(lm.is_logged_on()))
    return lm.is_logged_on()


def connect(url=DKRZ_ESG_URL):
    """establish ESGF search connection."""
    return SearchConnection(url, distrib=True)


def context(attrs, conn=None, verbose=False):
    """get an ESGF search context."""
    if conn is None:
        ctx = connect().new_context(**attrs)
    else:
        ctx = conn.new_context(**attrs)
    if verbose:
        print("Hit Count: {}".format(ctx.hit_count))
    return ctx


def datasets(attrs, conn=None, verbose=False):
    """returns dataset results"""
    return context(attrs, conn, verbose).search()


def dataset_dict(attrs, conn=None, verbose=False):
    return {ds.dataset_id: ds for ds in tqdm(datasets(attrs, conn, verbose))}


def get_opendap_urls(attrs, conn=None, agg=False, verbose=False):
    """agg=True will return aggregation urls
    instead of list of file urls...
    """
    ctx = context(attrs, conn, verbose)
    # for facet, item in ctx.facet_counts.items():
    #    if item: print('{} : {}'.format(facet, item))
    results = ctx.search()
    open_dap = {}
    for result in tqdm(results):
        sources = []
        if verbose:
            print(result.dataset_id)
        if agg:
            ctx_results = result.aggregation_context().search()
        else:
            ctx_results = result.file_context().search()
        for ctx_result in ctx_results:
            sources.append(ctx_result.opendap_url)
        open_dap[result.dataset_id] = sources
    if verbose:
        print("found {} datasets".format(len(open_dap.keys())))
    return open_dap


def _split_dataset_id(dataset_id):
    """splits a dataset id into list of values."""
    return dataset_id.split("|")[0].split(".") + [(dataset_id.split("|")[1])]


def _get_columns(columns=None):
    if columns == "CORDEX":
        return CORDEX_COLUMNS
    elif columns == "CMIP5":
        return CMIP5_COLUMNS
    elif columns == "CMIP6":
        return CMIP6_COLUMNS
    else:
        return columns


def to_pandas(results, columns=None):
    """convert a search result to dataframe"""
    data = [_split_dataset_id(res.dataset_id) for res in tqdm(results)]
    return pd.DataFrame(data, columns=_get_columns(columns))



# -*- coding: utf-8 -*-
# flake8: noqa
"""Variable module

This module defines variables for the CORDEX data request.

"""

from .tables import variables as TABLES


def table(name):
    """Top level function that returns a CORDEX data request table.

    Args:
      name (str): name of the CORDEX table.

    Returns:
      table (DataFrame): Cordex table.

    """
    return TABLES[name]


def tables():
    """Top level function that returns a list of available CORDEX data request tables.

    Returns:
      names (list): list of available CORDEX data request tables.

    """
    return list(TABLES.keys())


class Variable:
    """The :class:`Variable` holds data and meta information of a Cordex Variable.


    **Attributes:**
        *variable_id:*
            cordex variable id.
        *project_id:*
            cordex project id.

    """

    def __init__(self, variable_id, project_id="cmip5"):
        self.variable_id = variable_id
        self.project_id = project_id
        self._from_table()

    def __str__(self):
        return str(self.series)

    def __repr__(self):
        return str(self.series)

    def __getattr__(self, attr):
        if attr in self.series:
            return self.series[attr]
        else:
            raise AttributeError

    def _from_table(self):
        self.series = table(self.project_id).loc[self.variable_id]


def variables(project_id="cmip5"):
    """Top level function that returns all CORDEX variables.

    Args:
      project_id (str): project_id.

    Returns:
      variables (dict): dictionary of Cordex variables.

    """
    return {id: Variable(id, project_id) for id in table(project_id).index}


def variable(variable_id, project_id="cmip5"):
    """Top level function that returns a CORDEX variable.

    Args:
      variable_id (str): name of the variable.

    Returns:
      variable (:class:`_Variable`): Cordex variable.

    """
    return Variable(variable_id, project_id)



"""convert to WGS84 latitude-longitude projection"""
WGS84 = "EPSG:4326"


def get_geodataframe(shapefile, to_crs=WGS84, **kwargs):
    import geopandas as gpd

    shp = gpd.read_file(shapefile)
    if to_crs is not None:
        shp = shp.to_crs(to_crs)
    return shp


def get_regionmask(geodataframe, **kwargs):
    import regionmask

    return regionmask.from_geopandas(geodataframe, **kwargs)



# -*- coding: utf-8 -*-
# from pkg_resources import get_distribution, DistributionNotFound
#
# try:
#    # Change here if project is renamed and does not equal the package name
#    dist_name = "py-cordex"
#    __version__ = get_distribution(dist_name).version
# except DistributionNotFound:
#    __version__ = "unknown"
# finally:
#    del get_distribution, DistributionNotFound


from . import core, regions
from .core.domain import (
    cordex_domain,
    create_dataset,
    rotated_coord_transform,
    map_crs,
    domain_info,
)

from .tables import domains, ecmwf

from .version import version

__version__ = version



import pandas as pd

from ._resources import read_cordex_domain_tables, ecmwf_tables


class read_cls:
    def __init__(self, reader):
        self.reader = reader

    @property
    def tables(self):
        return self.reader()

    @property
    def table(self):
        return pd.concat(self.tables.values())

    # def __getattr__(self, table):
    #    return self.tables[table]


domains = read_cls(read_cordex_domain_tables)

ecmwf = read_cls(ecmwf_tables)



# -*- coding: utf-8 -*-
# flake8: noqa
"""Domain module

This module defines preconfigured CORDEX domain from csv tables. The module
also contains some tools to create a domain dataset from a csv tables or simply
from grid information.

Example:

    To get a list of available implementations, create cordex domains, write
    them to netcdf with some dummy data, you can use ,e.g.,::

        from cordex import domain as dm

        eur11 = dm.cordex_domain('EUR-11')

"""

import numpy as np
import pandas as pd
import xarray as xr

from ..tables import domains
from . import cf
from . import utils


def domain_names(table_name=None):
    """Returns a list of short names of all availabe Cordex domains

    Parameters
    ----------
    table_name:
        Only return domain names from this table.

    Returns
    -------
    domain names : list
        List of available cordex domains.

    """
    if table_name:
        return list(domains.tables[table_name].index)
    else:
        return list(domains.table.index)


def cordex_domain(short_name, dummy=False, tables=None, attrs=None):
    """Creates an xarray dataset containg the domain grid definitions.

    Parameters
    ----------
    short_name:
        Name of the Cordex Domain.
    dummy : str or logical
        Name of dummy field, if dummy=topo, the cdo topo operator will be
        used to create some dummy topography data. dummy data is useful for
        looking at the domain with ncview.
    tables: dataframe or list of dataframes, default: cordex_tables
        Tables from which to look up the grid information. Index in the table
        should be the short name of the domain, e.g., `EUR-11`. If no table is
        provided, all standard tables will be searched.
    attrs: str or dict
        Global attributes that should be added to the dataset. If `attrs='CORDEX'`
        a set of standard CF global attributes.

    Returns
    -------
    Dataset : xarray.core.Dataset
        Dataset containing the coordinates.

    Example
    -------

    To create a cordex rotated pole domain dataset, you can use ,e.g.,::

        import cordex as cx

        eur11 = cx.cordex_domain('EUR-11')

    """
    if attrs is None:
        attrs = {}
    if tables is None:
        tables = domains.table
    if isinstance(tables, list):
        config = pd.concat(tables).loc[short_name]
    else:
        config = tables.loc[short_name]
    return create_dataset(**config, name=short_name, dummy=dummy, attrs=attrs)


def create_dataset(
    nlon,
    nlat,
    dlon,
    dlat,
    ll_lon,
    ll_lat,
    pollon,
    pollat,
    name=None,
    dummy=False,
    attrs=None,
    **kwargs
):
    """Create domain dataset from grid information.

    Parameters
    ----------
    nlon : int
        longitudal number of grid boxes
    nlat : int
        latitudal number of grid boxes
    dlon : float
        longitudal resolution (degrees)
    dlat : float
        latitudal resolution (degrees)
    ll_lon : float
        lower left rotated longitude (degrees)
    ll_lat : float
        lower left rotated latitude (degrees)
    pollon : float
        pol longitude (degrees)
    pollat : float
        pol latitude (degrees)
    dummy : str or logical
        Name of dummy field, if dummy=topo, the cdo topo operator will be
        used to create some dummy topography data. dummy data is useful for
        looking at the domain with ncview.
    attrs: str or dict
        Global attributes that should be added to the dataset. If `attrs='CORDEX'`
        a set of standard CF global attributes.
    """
    if attrs == "CORDEX":
        attrs = cf.DEFAULT_CORDEX_ATTRS
    elif attrs is None:
        attrs = {}
    if name:
        attrs["CORDEX_domain"] = name
    rlon, rlat = _init_grid(nlon, nlat, dlon, dlat, ll_lon, ll_lat)
    lon, lat = rotated_coord_transform(*_stack(rlon, rlat), pollon, pollat)
    pole = _grid_mapping(pollon, pollat)
    return _get_dataset(rlon, rlat, lon, lat, pole, dummy=dummy, attrs=attrs)


def domain_info(short_name, tables=None):
    """Returns a dictionary containg the domain grid definitions.

    Returns a dictionary with grid information according to the
    Cordex archive specifications.

    See https://is-enes-data.github.io/cordex_archive_specifications.pdf

    Parameters
    ----------
    short_name:
        Name of the Cordex Domain.

    Returns
    -------
    domain info : dict
        Dictionary containing the grid information.

    """
    if tables is None:
        tables = domains.table
    if isinstance(tables, list):
        config = pd.concat(tables).loc[short_name]
    else:
        config = tables.loc[short_name]
    return {**{"short_name": short_name}, **dict(**config)}


def _get_dataset(rlon, rlat, lon, lat, pole, dummy=None, mapping_name=None, attrs=None):
    if mapping_name is None:
        mapping_name = cf.DEFAULT_MAPPING_NCVAR
    data_vars = {mapping_name: pole}

    ds = xr.Dataset(
        data_vars=data_vars,
        coords=dict(
            rlon=(["rlon"], rlon),
            rlat=(["rlat"], rlat),
            lon=(["rlat", "rlon"], lon),
            lat=(["rlat", "rlon"], lat),
        ),
        attrs=attrs,
    )

    for key, coord in ds.coords.items():
        coord.encoding["_FillValue"] = None
        coord.attrs = cf.coords[key]

    if dummy:
        if dummy is True:
            dummy_name = "dummy"
        else:
            dummy_name = dummy
        dummy = xr.DataArray(
            data=np.zeros((ds.rlat.size, ds.rlon.size)),
            coords=(ds.rlat, ds.rlon),
        )
        dummy.attrs = {"grid_mapping": mapping_name, "coordinates": "lon lat"}
        ds[dummy_name] = dummy
        if dummy_name == "topo":
            # use cdo to create dummy topography data.
            from cdo import Cdo

            tmp = utils.get_tempfile()
            ds.to_netcdf(tmp)
            topo = Cdo().topo(tmp, returnXDataset=True)["topo"]
            ds[dummy_name] = xr.DataArray(
                data=topo.values,
                coords=(ds.rlat, ds.rlon),
            )
            ds[dummy_name].attrs.update(topo.attrs)
    return ds


def _grid_mapping(pollon, pollat, mapping_name=None):
    """creates a grid mapping DataArray object"""
    if mapping_name is None:
        mapping_name = cf.DEFAULT_MAPPING_NCVAR
    da = xr.DataArray(np.zeros((), dtype=np.int32))
    attrs = cf.mapping.copy()
    attrs["grid_north_pole_longitude"] = pollon
    attrs["grid_north_pole_latitude"] = pollat
    da.attrs = attrs
    return da


def _init_grid(nlon, nlat, dlon, dlat, ll_lon, ll_lat):
    """create coordinate arrays from lower left longitude and latitude"""
    rlon = np.array([ll_lon + i * dlon for i in range(0, nlon)], dtype=np.float64)
    rlat = np.array([ll_lat + i * dlat for i in range(0, nlat)], dtype=np.float64)
    return rlon, rlat


def _stack(x, y):
    """Stack 1d arrays into 2d fields."""
    tmp_x = np.array(x).squeeze()
    tmp_y = np.array(y).squeeze()
    return np.vstack(len(tmp_y) * (tmp_x,)), np.hstack(
        len(tmp_x) * (tmp_y[:, np.newaxis],)
    )


def rotated_coord_transform(lon, lat, np_lon, np_lat, direction="rot2geo"):
    """Transforms a coordinate into a rotated grid coordinate and vice versa.

    The coordinates have to be given in degree and will be returned in degree.

    Parameters
    ----------
    lon : float array like
        Longitude coordinate.
    lat : float array like
        Latitude coordinate.
    np_lon : float array like
        Longitude coordinate of the rotated north pole.
    np_lat : float array like
        Latitude coordinate of the rotated north pole.
    direction : str
        Direction of the rotation.
        Options are: 'rot2geo' (default) for a transformation to regular
        coordinates from rotated. 'geo2rot' transforms regular coordinates
        to rotated.

    Returns
    -------
    lon_new : array like
        New longitude coordinate.
    lat_new : array like
        New latitude coordinate.
    """

    # Convert degrees to radians
    lon = np.deg2rad(lon)
    lat = np.deg2rad(lat)

    theta = 90.0 - np_lat  # Rotation around y-axis
    phi = np_lon + 180.0  # Rotation around z-axis

    # Convert degrees to radians
    phi = np.deg2rad(phi)
    theta = np.deg2rad(theta)

    # Convert from spherical to cartesian coordinates
    x = np.cos(lon) * np.cos(lat)
    y = np.sin(lon) * np.cos(lat)
    z = np.sin(lat)

    # Regular -> Rotated
    if direction == "geo2rot":

        x_new = (
            np.cos(theta) * np.cos(phi) * x
            + np.cos(theta) * np.sin(phi) * y
            + np.sin(theta) * z
        )
        y_new = -np.sin(phi) * x + np.cos(phi) * y
        z_new = (
            -np.sin(theta) * np.cos(phi) * x
            - np.sin(theta) * np.sin(phi) * y
            + np.cos(theta) * z
        )

    # Rotated -> Regular
    elif direction == "rot2geo":

        phi = -phi
        theta = -theta

        x_new = (
            np.cos(theta) * np.cos(phi) * x
            + np.sin(phi) * y
            + np.sin(theta) * np.cos(phi) * z
        )
        y_new = (
            -np.cos(theta) * np.sin(phi) * x
            + np.cos(phi) * y
            - np.sin(theta) * np.sin(phi) * z
        )
        z_new = -np.sin(theta) * x + np.cos(theta) * z

    # Convert cartesian back to spherical coordinates
    lon_new = np.arctan2(y_new, x_new)
    lat_new = np.arcsin(z_new)

    # Convert radians back to degrees
    lon_new = np.rad2deg(lon_new)
    lat_new = np.rad2deg(lat_new)

    return lon_new, lat_new


def map_crs(lon, lat, projection, transform=None):
    """coordinate transformation of longitude and latitude

    Transforms the coordinates lat, lon from the transform crs
    into the projection crs using cartopy.crs.

    Parameters
    ----------
    lon : float array like
        Longitude coordinate.
    lat : float array like
        Latitude coordinate.
    projection : cartopy.crs
        Target coordinate reference system into which lat and lon
        should be projected.
    transform : cartopy.crs
        Source coordinate reference system in which lat and lon
        are defined.

    Returns
    -------
    lon : array like
        Projected longitude coordinate.
    lat : array like
        Projected latitude coordinate.

    """

    from cartopy import crs as ccrs

    if transform is None:
        transform = ccrs.PlateCarree()
    latlon = ccrs.PlateCarree()
    lon_stack, lat_stack = xr.broadcast(lon, lat)
    result = latlon.transform_points(projection, lon_stack.values, lat_stack.values)
    return result[:, :, 0], result[:, :, 1]



from . import data

"""convert to WGS84 latitude-longitude projection"""
WGS84 = "EPSG:4326"


def get_geodataframe(shapefile, to_crs=WGS84, **kwargs):
    import geopandas as gpd

    shp = gpd.read_file(shapefile)
    if to_crs is not None:
        shp = shp.to_crs(to_crs)
    return shp


def get_regionmask(geodataframe, **kwargs):
    import regionmask

    return regionmask.from_geopandas(geodataframe, **kwargs)


class VG2500:
    """VG2500 Deutschland Verwaltungsgrenzen

    ADE Administrative Ebene
        Werteübersicht: 1 = Staat 2 = Land 3 = Regierungsbezirk 4 = Kreis
    ARS Amtlicher Regionalschlüssel (bisher Attribut RS)
        Bei diesem Schlüssel handelt es sich um den statistischen Schlüssel. Der Schlüssel ist hierarchisch
        strukturiert und spiegelt die in der Bundesrepublik Deutschland bestehenden Verwaltungsebenen wider.
        Der ARS gliedert sich wie folgt:
        1. – 2. Stelle = Kennzahl des Landes
        3. Stelle = Kennzahl des Regierungsbezirks
        4. – 5. Stelle = Kennzahl des Kreises
        6. – 9. Stelle = Kennzahl der Verwaltungsgemeinschaft
        10. – 12. Stelle = Kennzahl der Gemeinde
    GEN Geografischer Name
    ARS_0 aufgefüllter Amtlicher Regionalschlüssel (bisher Attribut RS_0)
        grundsätzlich 12-stelliger ARS (mit Nullen rechtsseitig aufgefüllt)
    """

    @staticmethod
    def _filename(domain):
        # url = "https://daten.gdz.bkg.bund.de/produkte/vg/vg2500/aktuell/vg2500_01-01.gk3.shape.zip"
        shp_file = "!vg2500_01-01.gk3.shape/vg2500/vg2500_{}.shp".format(domain)
        fname = data.fetch_vg2500()
        return "zip://" + fname + shp_file

    @classmethod
    def geodata(cls, domain="lan"):
        """Returns a GeoDataFrame object."""
        url = cls._filename(domain)
        geodata = get_geodataframe(url)
        geodata["name"] = geodata["ARS"] + "_" + geodata["GEN"]
        return geodata

    @classmethod
    def regionmask(cls, domain="lan"):
        """Returns a maks."""
        return get_regionmask(cls.geodata(domain), names="name", abbrevs="_from_name")



from pooch import retrieve
import pandas
import pooch

cache_url = "~/.py-cordex"


def fetch_vg2500():
    """Fetch Germany Verwaltungsgebiete 1:2,500,000"""
    fname = retrieve(
        path=cache_url,
        url="https://daten.gdz.bkg.bund.de/produkte/vg/vg2500/2020/vg2500_01-01.gk3.shape.zip",
        known_hash="md5:5a1a86cd131decd9cf116dbfc1a66f17",
    )
    return fname


def fetch_prudence():
    """Fetch Prudence regions table"""
    fname = retrieve(
        path=cache_url,
        url="https://raw.githubusercontent.com/euro-cordex/tables/master/regions/prudence.csv",
        known_hash="d87691a873110c9e3e4460a0ed35cd15f11f2a42aa86aced76feae9e87e8bed2",
    )
    return fname



import pandas as pd
import pooch


base_url = "https://raw.githubusercontent.com/euro-cordex/tables/master/"

cache_url = "~/.py-cordex"

DOMAIN_RESOURCE = pooch.create(
    # Use the default cache folder for the OS
    path=cache_url,  # pooch.os_cache("cordex"),
    # The remote data is on Github
    base_url=base_url + "domains/",
    registry={
        "cordex.csv": "115acce1612ff7d0908ba6693d3ef3892ee01f8656239b7383abdd633a3cbd7d",
        "cordex-high-res.csv": "a9b69a4a0db7e0b5920fc5ae4b030b770697f33e993f4b8fe39bd3a8093bad7c",
        "cordex-fps.csv": "09185a260a505c64edf1970a0c7ae91f5adb4d6f08d7e7af83767b96bd5017e3",
        "cordex-core.csv": "fd81e159ae5540581424c2f43a462b324e37b3a2cd7deddc2cf415c077fd8c2b",
    },
)


REGION_RESOURCE = pooch.create(
    # Use the default cache folder for the OS
    path=cache_url,  # pooch.os_cache("cordex"),
    # The remote data is on Github
    base_url=base_url + "regions/",
    registry={
        "prudence.csv": "d87691a873110c9e3e4460a0ed35cd15f11f2a42aa86aced76feae9e87e8bed2",
    },
)


ECMWF_RESOURCE = pooch.create(
    # Use the default cache folder for the OS
    path=cache_url,  # pooch.os_cache("cordex"),
    # The remote data is on Github
    base_url=base_url + "ecmwf/",
    registry={
        "ecmwf_128.csv": "7c29b13c2d74b9442b532268f7f4ed9eaf507cc21f4dc78242495bb387ce6ff4",
    },
)


def fetch_remote_table(name, resource):
    """
    uses pooch to cache files
    """

    # the file will be downloaded automatically the first time this is run.
    return resource.fetch(name)


def read_remote_table(name, resource, index_col=None):

    fname = fetch_remote_table(name, resource)

    return pd.read_csv(fname, index_col=index_col)


def read_region_table(name):

    return read_remote_table(name, resource=REGION_RESOURCE, index_col="area")


def read_cordex_domain_tables():

    resource = DOMAIN_RESOURCE
    return {
        table.split(".")[0]: read_remote_table(table, resource, index_col="short_name")
        for table in resource.registry.keys()
    }


def region_tables():

    return {
        table.split(".")[0]: read_region_table(table)
        for table in REGION_RESOURCE.registry.keys()
    }


def ecmwf_tables():

    resource = ECMWF_RESOURCE
    return {
        table.split(".")[0]: read_remote_table(table, resource, index_col="code")
        for table in ECMWF_RESOURCE.registry.keys()
    }



# -*- coding: utf-8 -*-
# flake8: noqa

import numpy as np
import pytest

import cordex as cx

from . import has_cartopy, requires_cartopy


def test_constructor():
    eur11 = cx.cordex_domain("EUR-11")
    eur11_user = cx.create_dataset(
        nlon=424,
        nlat=412,
        dlon=0.11,
        dlat=0.11,
        ll_lon=-28.375,
        ll_lat=-23.375,
        pollon=-162.00,
        pollat=39.25,
    )
    assert eur11_user.equals(eur11)


def test_domain_info():
    import pandas as pd

    info = {
        "short_name": "EUR-11",
        "region": 4,
        "long_name": "Europe",
        "nlon": 424,
        "nlat": 412,
        "ll_lon": -28.375,
        "ur_lon": 18.155,
        "ll_lat": -23.375,
        "ur_lat": 21.835,
        "dlon": 0.11,
        "dlat": 0.11,
        "pollon": -162.0,
        "pollat": 39.25,
    }
    table = pd.DataFrame.from_dict(
        {key: [item] for key, item in info.items()}
    ).set_index("short_name")
    assert cx.domain_info("EUR-11", table) == info


# @requires_cartopy
def test_mapping():
    import cartopy.crs as ccrs

    eur11 = cx.cordex_domain("EUR-11")
    pole = (
        eur11.rotated_latitude_longitude.grid_north_pole_longitude,
        eur11.rotated_latitude_longitude.grid_north_pole_latitude,
    )
    lon1, lat1 = cx.rotated_coord_transform(
        eur11.rlon, eur11.rlat, *pole, direction="rot2geo"
    )
    transform = ccrs.RotatedPole(-162.0, 39.25)
    lon2, lat2 = cx.map_crs(eur11.rlon, eur11.rlat, transform)

    assert np.allclose(lon1, lon2)
    assert np.allclose(lat1, lat2)



#!/usr/bin/env python
from setuptools import setup

# get version
with open("cordex/version.py") as f:
    line = f.readline().strip().replace(" ", "").replace('"', "")
    version = line.split("=")[1]
    __version__ = version

setup(version=__version__)
